from __future__ import annotations

from typing import (
    Any,
    Dict,
    Iterable,
    List,
    Optional,
    Tuple,
    Type,
    Union,
    cast,
)

from aiohttp import BasicAuth, ClientSession, TCPConnector

from ai_openchat.base.model import Model
from ai_openchat.base.session import BaseSession

_ProxyBasic = Union[str, Tuple[str, BasicAuth]]
_ProxyChain = Iterable[_ProxyBasic]
_ProxyType = Union[_ProxyChain, _ProxyBasic]


def _retrieve_basic(basic: _ProxyBasic) -> Dict[str, Any]:
    from aiohttp_socks.utils import parse_proxy_url  # type: ignore

    proxy_auth: Optional[BasicAuth] = None

    if isinstance(basic, str):
        proxy_url = basic
    else:
        proxy_url, proxy_auth = basic

    proxy_type, host, port, username, password = parse_proxy_url(proxy_url)
    if isinstance(proxy_auth, BasicAuth):
        username = proxy_auth.login
        password = proxy_auth.password

    return dict(
        proxy_type=proxy_type,
        host=host,
        port=port,
        username=username,
        password=password,
        rdns=True,
    )


def _prepare_connector(chain_or_plain: _ProxyType) -> Tuple[Type["TCPConnector"], Dict[str, Any]]:
    from aiohttp_socks import ChainProxyConnector, ProxyConnector, ProxyInfo  # type: ignore

    # since tuple is Iterable(compatible with _ProxyChain) object, we assume that
    # user wants chained proxies if tuple is a pair of string(url) and BasicAuth
    if isinstance(chain_or_plain, str) or (
            isinstance(chain_or_plain, tuple) and len(chain_or_plain) == 2
    ):
        chain_or_plain = cast(_ProxyBasic, chain_or_plain)
        return ProxyConnector, _retrieve_basic(chain_or_plain)

    chain_or_plain = cast(_ProxyChain, chain_or_plain)
    infos: List[ProxyInfo] = []
    for basic in chain_or_plain:
        infos.append(ProxyInfo(**_retrieve_basic(basic)))

    return ChainProxyConnector, dict(proxy_infos=infos)


class AiohttpSession(BaseSession):
    def __init__(self, proxy: Optional[_ProxyType] = None, **kwargs: Any) -> None:
        super().__init__(**kwargs)

        self._session: Optional[ClientSession] = None
        self._connector_type: Type[TCPConnector] = TCPConnector
        self._connector_init: Dict[str, Any] = {}
        self._should_reset_connector = True  # flag determines connector state
        self._proxy: Optional[_ProxyType] = None

        if proxy is not None:
            try:
                self._setup_proxy_connector(proxy)
            except ImportError as exc:  # pragma: no cover
                raise RuntimeError(
                    "In order to use aiohttp client for proxy requests, install "
                    "https://pypi.org/project/aiohttp-socks/"
                ) from exc

    def _setup_proxy_connector(self, proxy: _ProxyType) -> None:
        self._connector_type, self._connector_init = _prepare_connector(proxy)
        self._proxy = proxy

    @property
    def proxy(self) -> Optional[_ProxyType]:
        return self._proxy

    @proxy.setter
    def proxy(self, proxy: _ProxyType) -> None:
        self._setup_proxy_connector(proxy)
        self._should_reset_connector = True

    async def create_session(self) -> ClientSession:
        if self._should_reset_connector:
            await self.close()

        if self._session is None or self._session.closed:
            self._session = ClientSession(connector=self._connector_type(**self._connector_init))
            self._should_reset_connector = False

        return self._session

    async def close(self) -> None:
        if self._session is not None and not self._session.closed:
            await self._session.close()

    async def make_request(self, url, headers: dict, model: Model, timeout: Optional[int] = None):
        pass

    async def __aenter__(self) -> AiohttpSession:
        await self.create_session()
        return self



import json
from typing import Optional

from ai_openchat.aiiohttp import AiohttpSession
from ai_openchat.base.model import Model
from ai_openchat.base.session import BaseSession
from ai_openchat.utils.token import validate_token


class AsyncOpenAI:
    def __init__(self, token: str, session: Optional[BaseSession] = None) -> None:

        validate_token(token)

        if session is None:
            session = AiohttpSession()

        self.session = session
        self.__token = token

    @property
    def token(self) -> str:
        return self.__token

    async def send_message(self, message, model: Model):
        session = await self.session.create_session()
        url = self.session.api.api_url()
        headers = {'Content-Type': 'application/json', 'Authorization': 'Bearer ' + self.__token}
        model.prompt = message

        payload = json.dumps(model.__dict__)

        try:
            async with session.post(url, data=payload, headers=headers) as resp:
                return await resp.json()
        finally:
            await session.close()



#!/usr/bin/env python

from io import open
from setuptools import setup

"""
:authors: Whitev2
:license: Apache License, Version 2.0, see LICENSE file
:copyright: (c) 2023 Whitev2
"""

version = '1.0.2'

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='ai_openchat',
    version=version,

    author='Whitev2',
    author_email='maksfundd@gmail.com',

    description=(
        u'Python module for asynchronous interaction with OpenAI '
    ),
    long_description=long_description,
    long_description_content_type='text/markdown',

    url='https://github.com/Whitev2/async-openai',
    download_url='https://github.com/Whitev2/async-openai/archive/v{}/main.zip'.format(version),

    license='Apache License, Version 2.0, see LICENSE file',

    packages=['ai_openchat'],
    install_requires=['aiohttp>=3.8.3'],

    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python :: Implementation :: CPython',
    ]
)